@@ -326,6 +326,12 @@ fn parse_coverage_mapping<'data, R: ReadRef<'data>>(
     }
 }
 
+// Strictly speaking, parsing __llvm_covfun ought to depend on the version number
+// observed in the __llvm_covmap headers.
+//
+// But in practice there haven’t been any non-additive changes to the covfun format
+// in a long time, which is why you can get away with parsing them independently,
+// at least for now.
 fn parse_coverage_functions<'data, R: ReadRef<'data>>(
     endian: Endianness,
     section: &Section<'data, '_, R>,
@@ -423,6 +429,8 @@ fn parse_mapping_regions<'a>(
         bytes = data;
         let mut last_line = 0;
         for _ in 0..regions_len {
+            let mut mcdc_params = None;
+
             let mut false_count = Counter::default();
             let mut kind = RegionKind::Code;
             let (data, raw_header) = parse_leb128(bytes)?;
@@ -434,7 +442,7 @@ fn parse_mapping_regions<'a>(
                     kind = RegionKind::Expansion;
                     expanded_file_id = raw_header >> Counter::ENCODING_TAG_AND_EXP_REGION_BITS;
                     if expanded_file_id >= file_indices.len() as u64 {
-                        todo!()
+                        panic!("expanded_file_id is invalid");
                     }
                 } else {
                     let shifted_counter = raw_header >> Counter::ENCODING_TAG_AND_EXP_REGION_BITS;
@@ -449,6 +457,36 @@ fn parse_mapping_regions<'a>(
                             false_count = parse_counter(c2, expressions);
                             bytes = data;
                         }
+                        Ok(RegionKind::MCDCBranch) => {
+                            kind = RegionKind::MCDCBranch;
+                            let (data, c1) = parse_leb128(bytes)?;
+                            let (data, c2) = parse_leb128(data)?;
+
+                            counter = parse_counter(c1, expressions);
+                            false_count = parse_counter(c2, expressions);
+
+                            let (data, id1) = parse_leb128(data)?;
+                            let (data, tid1) = parse_leb128(data)?;
+                            let (data, fid1) = parse_leb128(data)?;
+                            bytes = data;
+
+                            mcdc_params = Some(MCDCParams::Branch(BranchParameters {
+                                id: id1.try_into().unwrap(),
+                                false_cond: fid1.try_into().unwrap(),
+                                true_cond: tid1.try_into().unwrap(),
+                            }));
+                        }
+                        Ok(RegionKind::MCDCDecision) => {
+                            kind = RegionKind::MCDCDecision;
+                            let (data, bidx) = parse_leb128(data)?;
+                            let (data, nc) = parse_leb128(data)?;
+                            bytes = data;
+
+                            mcdc_params = Some(MCDCParams::Decision(DecisionParameters {
+                                bitmap_idx: bidx.try_into().unwrap(),
+                                num_conditions: nc.try_into().unwrap(),
+                            }));
+                        }
                         e => panic!("Malformed: {:?}", e),
                     }
                 }
@@ -483,6 +521,7 @@ fn parse_mapping_regions<'a>(
                     column_start,
                     column_end,
                 },
+                mcdc_params,
             });
         }
     }
